#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/keysym.h>

#define  MYPI  3.141592653589793238462643383279502
#define maxsx 640
#define maxsy 384
#define maxpnt 2000
#define maxseg 2000
#define maxpol 500
#define maxnoeud 2000

/* definition des structures initiales
   utilisees lors de la definition des objets */

/* definition d'un vecteur de dimension 3 */
struct Vec
	{
	float	xv;
	float	yv;
	float	zv;
	};

typedef struct Vec	*vec;

/* definition d'une matrice 3*3 */
struct Mat
	{
	float	aa;
	float	ab;
	float	ac;
	float	ba;
	float	bb;
	float	bc;
	float	ca;
	float	cb;
	float	cc;
	};

typedef struct Mat	*mat;

/* definition d'un point de l'espace */
struct	Pnt
	{
	float	x;
	float	y;
	float	z;
	int	nxtp;
	};

typedef struct Pnt	*pnt;


/* definition d'un segment */
struct	Seg
	{
	int	n1;
	int	n2;
	};

typedef struct Seg *seg;

/* definition d'un polygone */
struct	Pol
	{
	int		type;
	struct Vec	norm;		/* vecteur normal */
	int		col;
	int		frere;		/* frere du polygone */
	int		ns;		/* nombre de sommets */
	int		nums[3];	/* numero des arretes */
	};

typedef struct Pol	*pol;

/* definition d'un noeud, pour les faces cachees */
struct Noeud
	{
	int	firstp;		/* no premier polygone */
	int	filsp;		/* no fils plus */
	int	filsm;		/* no fils moins */
	float	a,b,c,d;	/* plan test : ax+by+cz+d=0 */
	};

typedef struct Noeud	*noeud;

/* definition d'un groupe */
struct Group
	{
	pnt	pnt1;	/* premier point */
	int	npnt;	/* nombre de points */
	seg	seg1;
	int	nseg;
	pol	pol1;
	int	npol;
	};

typedef struct Group	*group;


/* definition d'un groupe de passage */
struct Pass
	{
	struct Mat	m;	/* matrice de passage B->A */
	struct Vec	v;	/* vecteur de passage exprime dans A */
	};

typedef struct Pass	*pass;


/* definition d'un objet 3d */
struct Obj3d
	{
	struct Pass	p;		/* donnees de passage */
	group		grp;		/* groupe attache (NULL:aucun)*/
	struct Obj3d	*pere;		/* objet pere (Null:root) */
	struct Obj3d    *frere;		/* prochain frere (Null:termine) */
	struct Obj3d    *fils1;		/* premier fils (Null:pas de fils) */
	};

typedef struct Obj3d	*obj3d;



/* definition des structures de travail
   utilisees au cours du calcul */

/* definition d'un point */
struct Pntw
	{
	struct Pnt	p;		/* point dans repere obs */
	int		state;		/* rapport avec la pyramide */
	float		xp;		/* coordonnees ecran */
	float		yp;
	float		zphi;		/* intermediaires de calcul */
	float		zpsi;
	};

typedef struct Pntw	*pntw;

/* definition d'un polygone */
struct Polw
	{
	int		type;		/* type de polygone */
	struct Vec	norm;		/* vecteur normal dans le repere obs */
	float		oan;		/* produit OA.n */
	int		col;		/* couleur */
	int		frere;		/* frere du polygone */
	int		np;		/* nombre de sommets */
	int		pn[8];		/* numeros des sommets */
	};

typedef struct Polw	*polw;


/* definition de l'observateur */
struct Observ
	{
	Display	*display;
	Pixmap	dessin;
	GC	gc;
	obj3d	pos;		/* objet lie a l'observateur */
	float	dist;		/* distance de l'ecran */
	float	sx;		/* demi-largeur de l'ecran */
	float	sy;		/* demi-hauteur de l'ecran */
	float	phi;		/* phi=xmax/dist */
	float	psi;		/* psi=ymax/dist */
	};

typedef struct Observ	*observ;

/* definition d'un mode tortue */
struct Turtle
	{
	int	dx;
	int	dy;
	int	dz;
	int	da;
	int	db;
	int	dc;
	int	rx;
	int	ry;
	int	rz;
	};

typedef struct Turtle	*turtle;

/* definition rotation spherique */
struct Rotsph
	{
	float	th;
	float	ph;
	float	rho;
	};

typedef struct Rotsph	*rotsph;

typedef unsigned long PIXEL;

extern int	black,white;
extern PIXEL	*pixels;	

/* calcule une image 3d a partir des caracteristiques de l'observateur,
   des buffers points, segments, polygones et bitmap */

void image3d(observ obs, pass passobs, int modegraph);

/* initialise les donnees de l'espace */

int inidata(char *name);

/* deplace l'observateur selon la souris */
void deplace(observ obs, pass pb, int dx, int dy, unsigned int state
		, int modedep, rotsph rsph);

/* routines generales de calcul matriciel */


/* multiplie le vecteur v par le reel k -> r (r peut etre egal a v) */
void mulscalvec(float k, vec v, vec r);

/* multiplie la matrice m par le reel k -> r (r peut etre egal a m) */
void mulscalmat(float k, mat m, mat r);


/* calcule le produit m.v -> r (r peut etre egal a v) */
void mulmatvec(mat m, vec v, vec r);

/* calcule le produit tm.v -> r (r peut etre egal a v) */
void tmulmatvec(mat m, vec v, vec r);

/* calcule le produit m.n -> r (r peut etre egal a m ou n)*/
void mulmatmat(mat m, mat n, mat r);


/* calcule le vecteur normal et norme pq^pr -> v*/
void vnorm(pnt p, pnt q, pnt r, vec v);

/* calcule la matrice de passage correspondant a rsph */
void convmat(rotsph rsph, pass obs);


