#include "3d.h"
#define un	1.0
#define cosa	0.999961923064
#define sina	0.00872653549837

struct Mat mxp={un,0,0,0,cosa,sina,0,-sina,cosa};
struct Mat mxm={un,0,0,0,cosa,-sina,0,sina,cosa};
struct Mat myp={cosa,0,sina,0,un,0,-sina,0,cosa};
struct Mat mym={cosa,0,-sina,0,un,0,sina,0,cosa};
struct Mat mzp={cosa,sina,0,-sina,cosa,0,0,0,un};
struct Mat mzm={cosa,-sina,0,sina,cosa,0,0,0,un};

/* multiplie le vecteur v par le reel k -> r (r peut etre egal a v) */
void mulscalvec(float k, vec v, vec r)
	{
	r->xv=k*v->xv;
	r->yv=k*v->yv;
	r->zv=k*v->zv;
	}

/* multiplie la matrice m par le reel k -> r (r peut etre egal a m) */
void mulscalmat(float k, mat m, mat r)
	{
	r->aa=k*m->aa; r->ab=k*m->ab; r->ac=k*m->ac;
	r->ba=k*m->ba; r->bb=k*m->bb; r->bc=k*m->bc;
	r->ca=k*m->ca; r->cb=k*m->cb; r->cc=k*m->cc;
	}

/* calcule le produit m.v -> r (r peut etre egal a v) */
void mulmatvec(mat m, vec v, vec r)
	{
		struct Vec	p;
	p.xv=m->aa*v->xv + m->ab*v->yv + m->ac*v->zv;
	p.yv=m->ba*v->xv + m->bb*v->yv + m->bc*v->zv;
	p.zv=m->ca*v->xv + m->cb*v->yv + m->cc*v->zv;

	*r=p;
	}

/* calcule le produit tm.v -> r (r peut etre egal a v) */
void tmulmatvec(mat m, vec v, vec r)
	{
		struct Vec	p;
	p.xv=m->aa*v->xv + m->ba*v->yv + m->ca*v->zv;
	p.yv=m->ab*v->xv + m->bb*v->yv + m->cb*v->zv;
	p.zv=m->ac*v->xv + m->bc*v->yv + m->cc*v->zv;

	*r=p;
	}

/* calcule le produit m.n -> r (r peut etre egal a m ou n)*/
void mulmatmat(mat m, mat n, mat r)
	{
		struct Mat	p;

	p.aa=m->aa*n->aa + m->ab*n->ba + m->ac*n->ca;
	p.ba=m->ba*n->aa + m->bb*n->ba + m->bc*n->ca;
	p.ca=m->ca*n->aa + m->cb*n->ba + m->cc*n->ca;

	p.ab=m->aa*n->ab + m->ab*n->bb + m->ac*n->cb;
	p.bb=m->ba*n->ab + m->bb*n->bb + m->bc*n->cb;
	p.cb=m->ca*n->ab + m->cb*n->bb + m->cc*n->cb;

	p.ac=m->aa*n->ac + m->ab*n->bc + m->ac*n->cc;
	p.bc=m->ba*n->ac + m->bb*n->bc + m->bc*n->cc;
	p.cc=m->ca*n->ac + m->cb*n->bc + m->cc*n->cc;

	*r=p;
	}

/* calcule le vecteur normal et norme pq^pr -> v*/
void vnorm(pnt p, pnt q, pnt r, vec v)
	{
		float	x1,y1,z1,x2,y2,z2,x,y,z,nm;

	x1=q->x - p->x;
	y1=q->y - p->y;
	z1=q->z - p->z;

	x2=r->x - p->x;
	y2=r->y - p->y;
	z2=r->z - p->z;

	x=y1*z2 - z1*y2;
	y=z1*x2 - x1*z2;
	z=x1*y2 - y1*x2;
	nm=sqrt(x*x+y*y+z*z);

	if (nm==0.0) nm=1.0;
	v->xv=x/nm;
	v->yv=y/nm;
	v->zv=z/nm;
	}

/* calcule la matrice de passage correspondant a rsph */
void convmat(rotsph rsph, pass obs)
	{
		float	ct,st,cp,sp;

	ct=cos(rsph->th); st=sin(rsph->th);
	cp=cos(rsph->ph); sp=sin(rsph->ph);
	obs->m.aa=-sp;		obs->m.ab=0;	obs->m.ac=cp;
	obs->m.ba=-st*cp;	obs->m.bb=ct;	obs->m.bc=-st*sp;
	obs->m.ca=-ct*cp;	obs->m.cb=-st;	obs->m.cc=-ct*sp;

	obs->v.xv=0; obs->v.yv=0; obs->v.zv=rsph->rho;
	}

void depx(pass pb,int dx)
	{
	pb->v.xv+=dx;
	}

void depy(pass pb,int dx)
	{
	pb->v.yv+=dx;
	}

void depz(pass pb,int dx)
	{
	pb->v.zv+=dx;
	}

void rotx(pass pb,int dx)
	{
		int	i;

	if (dx>0)
		for (i=0;i<dx;i++)
		{
		mulmatmat(&mxp,&pb->m,&pb->m);
		mulmatvec(&mxp,&pb->v,&pb->v);
		}
	if (dx<0)
		for (i=0;i<-dx;i++)
		{
		mulmatmat(&mxm,&pb->m,&pb->m);
		mulmatvec(&mxm,&pb->v,&pb->v);
		}
	}

void roty(pass pb,int dx)
	{
		int	i;

	if (dx>0)
		for (i=0;i<dx;i++)
		{
		mulmatmat(&myp,&pb->m,&pb->m);
		mulmatvec(&myp,&pb->v,&pb->v);
		}
	if (dx<0)
		for (i=0;i<-dx;i++)
		{
		mulmatmat(&mym,&pb->m,&pb->m);
		mulmatvec(&mym,&pb->v,&pb->v);
		}
	}

void rotz(pass pb,int dx)
	{
		int	i;

	if (dx>0)
		for (i=0;i<dx;i++)
		{
		mulmatmat(&mzp,&pb->m,&pb->m);
		mulmatvec(&mzp,&pb->v,&pb->v);
		}
	if (dx<0)
		for (i=0;i<-dx;i++)
		{
		mulmatmat(&mzm,&pb->m,&pb->m);
		mulmatvec(&mzm,&pb->v,&pb->v);
		}
	}

void rotth(rotsph rs, int dx)
	{
	rs->th+=dx/100.;
	if (rs->th>MYPI/2.) rs->th=MYPI/2.;
	if (rs->th<-MYPI/2.) rs->th=-MYPI/2.;
	}

void rotph(rotsph rs, int dx)
	{
	rs->ph+=dx/100.;
	}

void deprho(rotsph rs, int dx)
	{
	rs->rho+=dx;
	if (rs->rho<0.) rs->rho=0.;
	}

void chgdist(observ obs, int dx)
	{
	obs->dist+=dx;
	if (obs->dist<10) obs->dist=10;
	obs->phi=obs->sx/obs->dist;
	obs->psi=obs->sy/obs->dist;
	}

void deplace(observ obs, pass pb, int dx, int dy, unsigned int state
		, int modedep, rotsph rsph)
	{
	state&=Button1Mask|Button2Mask|Button3Mask;

switch (modedep)	{
	case 0 :
	
		switch (state)	{
		case Button1Mask:
			depx(pb,-dx);
			depy(pb,-dy);
			break;
		case Button2Mask:
			depx(pb,-dx);
			depz(pb,-dy);
			break;
		case Button3Mask:
			depz(pb,-dx);
			depy(pb,-dy);
			break;
		case Button2Mask|Button3Mask:
			roty(pb,-dy);
			depx(pb,-dx);
			break;
		case Button1Mask|Button3Mask:
			rotx(pb,-dx);
			depy(pb,-dy);
			break;
		case Button1Mask|Button2Mask:
			rotz(pb,-dx);
			chgdist(obs,-dy);
			break;
		default:
			break;
		}
		break;
	default:
		switch(state) {
		case Button1Mask :
			rotph(rsph,dx);
			rotth(rsph,dy);
			break;
		case Button2Mask :
			rotph(rsph,dx);
			deprho(rsph,-dy);
			break;
		case Button3Mask :
			chgdist(obs,-dx);
			rotth(rsph,dy);
			break;
		default:
			break;
		}
		convmat(rsph,pb);
		break;
		
	}
	

	}
